<?php

namespace Dj3c1t\AppBundle\Controller\Admin;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Form\FormError;

use Dj3c1t\AppBundle\Entity\User;

class UserController extends Controller {

    public function indexAction(Request $request) {
        $em = $this->get('doctrine')->getManager();
        $query = $em->createQuery(
            "SELECT u FROM Dj3c1tAppBundle:User u"
        );
        $pagination = $this->get('knp_paginator')->paginate(
            $query,
            $request->query->getInt('page', 1),
            10
        );
        return $this->render(
            'Dj3c1tAppBundle:Admin:User/index.html.twig',
            array(
                'pagination' => $pagination
            )
        );
    }

    public function addAction(Request $request) {
        $em = $this->get('doctrine')->getManager();
        $user = new User();
        $user->setIsActive(true);
        $form = $this->createForm(
            "dj3c1t_user",
            $user,
            array(
                'withAdminFields' => true,
            )
        );
        $form->remove('changePassword');
        $form->handleRequest($request);
        if($form->isValid()) {
            try {
                $factory = $this->get('security.encoder_factory');
                $encoder = $factory->getEncoder($user);
                $user->setPassword($encoder->encodePassword($user->getPassword(), $user->getSalt()));
                $user->setSalt("");
                $em->persist($user);
                $em->flush();
                $this->get('session')->getFlashBag()->add('success', "L'utilisateur a été ajouté");
                return $this->redirect($this->generateUrl('admin_user_list'));
            }
            catch(\Exception $e) {
                $this->get('session')->getFlashBag()->add('error', $e->getMessage());
            }
        }
        return $this->render(
            'Dj3c1tAppBundle:Admin:User/add.html.twig',
            array(
                "form" => $form->createView(),
            )
        );
    }

    public function editAction(Request $request, User $user) {
        $oldPassword = $user->getPassword();
        $form = $this->createForm(
            "dj3c1t_user",
            $user,
            array(
                'validation_groups' => array("edit"),
                'withAdminFields' => true,
            )
        );
        $form->handleRequest($request);
        if($form->isValid()) {
            if($form->get('changePassword')->getData()) {
                if($user->getPassword()) {
                    $encoder = $this->get('security.encoder_factory')->getEncoder($user);
                    $user->setPassword(
                        $encoder->encodePassword($user->getPassword(), $user->getSalt())
                    );
                }
                else {
                    $form->get('password')->addError(new FormError('Merci de préciser un mot de passe'));
                }
            }
            else {
                $user->setPassword($oldPassword);
            }
            if($form->isValid()) {
                try {
                    $em = $this->get('doctrine')->getManager();
                    $em->flush();
                    $this->get('session')->getFlashBag()->add('success', "L'utilisateur a été enregistré");
                    return $this->redirect(
                        $this->generateUrl(
                            'admin_user_edit',
                            array(
                                "user" => $user->getId(),
                            )
                        )
                    );
                }
                catch(\Exception $e) {
                    $this->get('session')->getFlashBag()->add('error', $e->getMessage());
                }
            }
        }
        return $this->render(
            'Dj3c1tAppBundle:Admin:User/edit.html.twig',
            array(
                "form" => $form->createView(),
            )
        );
    }

    public function toogleActiveAction(Request $request, User $user) {
        $em = $this->get('doctrine')->getManager();
        $user->setIsActive(!$user->getIsActive());
        try {
            $em->flush();
            $this->get('session')->getFlashBag()->add('success', "L'utilisateur a été ".($user->getIsActive() ? "activé" : "désactivé"));
        }
        catch(\Exception $e) {
            $this->get('session')->getFlashBag()->add('error', $e->getMessage());
        }
        return $this->redirect($this->generateUrl('admin_user_list'));
    }

    public function deleteAction(Request $request, User $user) {
        $em = $this->get('doctrine')->getManager();
        $em->remove($user);
        try {
            $em->flush();
            $this->get('session')->getFlashBag()->add('success', "L'utilisateur a été supprimé");
        }
        catch(\Exception $e) {
            $this->get('session')->getFlashBag()->add('error', $e->getMessage());
        }
        return $this->redirect($this->generateUrl('admin_user_list'));
    }

}
